@@ -2,6 +2,7 @@ import os
 import logging
 from laboro.error import LaboroError
 from laboro.history import History
+from laboro.workspace import Workspace
 from laboro.config.manager import Manager as ConfigMgr
 from laboro.module.manager import Manager as ModuleMgr
 from laboro.error.handler import Handler as ErrorHandler
@@ -10,7 +11,6 @@ from laboro.error.handler import Handler as ErrorHandler
 class Context:
   def __init__(self, logger):
     self.logger = logger
-    self.memory = dict()
     self.modules = dict()
     self.history = None
     self.modulemgr = ModuleMgr()
@@ -25,20 +25,34 @@ class Context:
                                                "$.laboro.log.dir")
     self.loglevel = self.configmgr.get_parameter("main",
                                                  "$.laboro.log.level")
+    self.workspace = None
     logging.getLogger().vault.clear()
 
   def reset(self, workflow_name, session):
+    logging.getLogger().log_section("WORKFLOW",
+                                    f"Started {workflow_name} / {session}")
     logging.getLogger().vault.clear()
-    self.memory = dict()
     self._configure_logger(workflow_name, session)
     self.set_workflow_config(workflow_name)
+    self.workspace = Workspace(workspacedir=self.workspacedir,
+                               workflow=workflow_name,
+                               session=session)
     self.history = History(filename=os.path.join(self.histdir,
                                                  f"{workflow_name}.db"),
                            workflow=workflow_name,
                            session=session,
                            params=self.configmgr.workflow_config)
+    self.history.enter()
+    self.install_packages()
     return self
 
+  def exit(self, session, kind, value, traceback, exit_code):
+    logging.info(f"[+] Workflow ended with code: {exit_code}")
+    self.workspace.delete()
+    self.history.exit(kind, value)
+    self.handle_error(kind, value, traceback)
+    self.logger.remove_file_handler(session)
+
   def _configure_logger(self, workflow_name, session):
     self.logger.add_file_handler(self.logdir, workflow_name, session)
     self.logger.set_log_level(self.loglevel)
@@ -72,7 +86,7 @@ class Context:
       secrets = [args[key] for key in args.keys() if key in secret_keys]
       list(map(logging.getLogger().vault.add, secrets))
     except IndexError as err:
-      raise LaboroError(f"UnknownMethodError: {action}") from err
+      raise LaboroError(f"UnknownMethodError: Unknown method {instance.__class__.__name__}.{action}") from err
 
   def handle_error(self, kind, value, traceback):
     ErrorHandler().handle_error(kind, value, traceback)
